# __init__.py
# Author: Thomas MINIER - MIT License 2019
"""
    Manipulate OTTR Reasonable Ontology Templates in Python
"""

from ottr.generator import OttrGenerator

__all__ = [
    'OttrGenerator'
]
